package com.iqball.app.page

import android.app.Activity
import android.content.res.Configuration
import android.util.Log
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalConfiguration
import arrow.core.Either
import arrow.core.flatMap
import com.iqball.app.component.BasketCourt
import com.iqball.app.model.TacticInfo
import com.iqball.app.model.tactic.CourtType
import com.iqball.app.model.tactic.StepContent
import com.iqball.app.model.tactic.StepNodeInfo
import com.iqball.app.net.service.TacticService
import com.iqball.app.session.Token
import kotlinx.coroutines.runBlocking
import java.time.Instant
import java.time.LocalDateTime
import java.time.ZoneId

private data class VisualizerInitialData(
    val info: TacticInfo,
    val rootStep: StepNodeInfo,
    val rootContent: StepContent
)

@Composable
fun VisualizerPage(
    service: TacticService,
    auth: Token,
    tacticId: Int,
) {
    val dataEither = initializeVisualizer(service, auth, tacticId)

    val (info, rootStep, rootContent) = when (dataEither) {
        is Either.Left -> return Text(text = dataEither.value)
        is Either.Right -> dataEither.value
    }

    val screenOrientation = LocalConfiguration.current.orientation

    Column {
        VisualizerHeader(title = info.name)
        when (screenOrientation) {
            Configuration.ORIENTATION_PORTRAIT -> Text(
                text = "Visualizing Tactic Steps tree",
                modifier = Modifier
                    .fillMaxSize()
                    .background(Color.Green)
            )

            Configuration.ORIENTATION_LANDSCAPE -> BasketCourt(
                content = rootContent,
                type = info.type
            )

            else -> throw Exception("Could not determine device's orientation.")
        }
    }

}

@Composable
private fun VisualizerHeader(title: String) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .background(Color.White),
        horizontalArrangement = Arrangement.SpaceBetween,
        verticalAlignment = Alignment.CenterVertically
    ) {
        IconButton(
            onClick = { /*TODO*/ }
        ) {
            Icon(
                imageVector = Icons.Filled.ArrowBack,
                contentDescription = "Back",
                tint = Color.Black
            )
        }

        Text(text = title, color = Color.Black)

        Text(text = "")
    }
}

private fun initializeVisualizer(
    service: TacticService,
    auth: Token,
    tacticId: Int
): Either<String, VisualizerInitialData> {
    val (tacticInfo, tacticTree, rootStepContent) = runBlocking {
        val tacticInfo = service.getTacticInfo(auth, tacticId)
            .map {
                TacticInfo(
                    id = it.id,
                    name = it.name,
                    type = CourtType.valueOf(
                        it.courtType.lowercase().replaceFirstChar(Char::uppercaseChar)
                    ),
                    creationDate = LocalDateTime.ofInstant(
                        Instant.ofEpochMilli(it.creationDate),
                        ZoneId.systemDefault()
                    )
                )
            }.onLeft {
                Log.e(
                    "received error response from server when retrieving tacticInfo : {}",
                    it.toString()
                )
            }

        val tacticTree = service.getTacticStepsTree(auth, tacticId)
            .map { it.root }
            .onLeft {
                Log.e(
                    "received error response from server when retrieving tactic steps tree: {}",
                    it.toString()
                )
            }

        val rootStepContent = tacticTree
            .flatMap {
                service.getTacticStepContent(auth, tacticId, it.id)
                    .onLeft {
                        Log.e(
                            "received error response from server when retrieving root content: {}",
                            it.toString()
                        )
                    }
            }

        Triple(tacticInfo.getOrNull(), tacticTree.getOrNull(), rootStepContent.getOrNull())
    }

    if (tacticInfo == null || tacticTree == null || rootStepContent == null) {
        return Either.Left("Unable to retrieve tactic information")
    }


    return Either.Right(VisualizerInitialData(tacticInfo, tacticTree, rootStepContent))
}